using System;
using System.Windows.Forms;
using SlimDX;
using SlimDX.DirectInput;

namespace PolledDevices
{
    /// <summary>
    /// 
    /// </summary>
	public class PolledKeyboard : PolledDevice
	{
		#region Fields

		private bool[] _oldDown;

        #endregion

        #region Events

        public event PolledKeyboardEventHandler	KeyboardPolledEvent;
		public event PolledKeyboardEventHandler	KeyboardDownEvent;
		public event PolledKeyboardEventHandler	KeyboardUpEvent;

		#endregion

		#region Constructor

        /// <summary>
        /// 
        /// </summary>
        /// <param name="owner"></param>
		public PolledKeyboard(Control owner) : base(owner)
		{
            //_oldDown = new bool[Key.MediaSelect - Key.Escape + 1];
            _oldDown = null;
		}

		#endregion

		#region PolledDevice Implementations

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
		protected override bool AcquireDevice()
		{
            _device = new SlimDX.DirectInput.Keyboard(_dinput);
            _device.SetCooperativeLevel(_owner, CooperativeLevel.Background | CooperativeLevel.Nonexclusive);
			_device.Acquire();

            KeyboardState s = ((Keyboard) _device).GetCurrentState();
            _oldDown = new bool[s.AllKeys.Count];
            for (int i = 0; i < _oldDown.Length; i++)
                _oldDown[i] = s.IsPressed(s.AllKeys[i]);

			return true;
		}

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
		protected override bool CompareState()
		{
			KeyboardState s;
			try
			{
                s = ((Keyboard) _device).GetCurrentState();
			}
            catch (SlimDX.DirectInput.DirectInputException)
            {
                return false;
            }

            bool[] newDown = null;
			if (KeyboardDownEvent != null || KeyboardUpEvent != null)
			{
                newDown = new bool[s.AllKeys.Count];
                for (int i = 0; i < newDown.Length; i++)
                    newDown[i] = s.IsPressed(s.AllKeys[i]);
			}

			//
			// fire events henceforth!!
			//

			if (KeyboardPolledEvent != null)
				KeyboardPolledEvent(this, new PolledKeyboardEventArgs(s));

            if (newDown != null)
			{
				for (int j = 0; j < _oldDown.Length; j++)
				{
					if (!_oldDown[j] && newDown[j]) // down
					{
						if (KeyboardDownEvent != null)
							KeyboardDownEvent(this, new PolledKeyboardEventArgs(s, s.AllKeys[j]));
					}
					else if (_oldDown[j] && !newDown[j]) // up
					{
						if (KeyboardUpEvent != null)
                            KeyboardUpEvent(this, new PolledKeyboardEventArgs(s, s.AllKeys[j]));
					}
				}
				Array.Copy(newDown, _oldDown, _oldDown.Length);
			}

			return true;
		}

		#endregion

		#region Other Methods

        /// <summary>
        /// 
        /// </summary>
        /// <param name="k"></param>
        /// <returns></returns>
		public int IsDown(Key k)
		{
			if (_device != null)
			{
                KeyboardState s = ((Keyboard) _device).GetCurrentState();
				return s.IsPressed(k) ? 1 : 0;
			}
            return -1; // error
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="k"></param>
        /// <returns></returns>
		public int IsUp(Key k)
		{
            if (_device != null)
            {
                KeyboardState s = ((Keyboard) _device).GetCurrentState();
                return s.IsReleased(k) ? 1 : 0;
            }
            return -1; // error
		}

		#endregion
	}
}
