using System;
using System.Drawing;
using System.Windows.Forms;
using SlimDX;
using SlimDX.DirectInput;

namespace PolledDevices
{
    /// <summary>
    /// 
    /// </summary>
	public class PolledMouse : PolledDevice
	{
		#region Fields & Events

		private MouseState _state;

        #endregion

        #region Events
        
        public event PolledMouseEventHandler MousePolledEvent;
		public event PolledMouseEventHandler MouseDownEvent;
        public event PolledMouseEventHandler MouseUpEvent;
        public event PolledMouseEventHandler MouseMoveEvent;

        #endregion

        #region Constructor

        /// <summary>
        /// 
        /// </summary>
        /// <param name="owner"></param>
        public PolledMouse(Control owner)
            : base(owner)
        {
            // do nothing
        }

		#endregion

		#region PolledDevice Implementations

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
		protected override bool AcquireDevice()
		{
            _device = new SlimDX.DirectInput.Mouse(_dinput);
			_device.SetCooperativeLevel(_owner, CooperativeLevel.Background | CooperativeLevel.Nonexclusive);
			_device.Acquire();
            _state = ((Mouse) _device).GetCurrentState();
			return true;
		}

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
		protected override bool CompareState()
		{
			MouseState newState;
			try
			{
                newState = ((Mouse) _device).GetCurrentState();
			}
			catch (SlimDX.DirectInput.DirectInputException)
			{
				return false;
			}

			Point screen = Cursor.Position;
            Point client = _owner.PointToClient(screen);
			
            // has the mouse moved?
            bool positionChanged = (newState.X != _state.X || newState.Y != _state.Y || newState.Z != _state.Z);
			
			// get the buttons for determining change
            bool[] oldButtons = _state.GetButtons();
            bool[] newButtons = newState.GetButtons();

			// copy the state reference before firing events
			_state = newState;

			//
			// fire events henceforth!
			//

			// generic polled event
			if (MousePolledEvent != null)
				MousePolledEvent(this, new PolledMouseEventArgs(_state, client, screen));

			// mouse movement
			if (positionChanged && MouseMoveEvent != null)
				MouseMoveEvent(this, new PolledMouseEventArgs(_state, client, screen));
			
			// mouse buttons
			if (MouseDownEvent != null || MouseUpEvent != null)
			{
				for (int i = 0; i < oldButtons.Length; i++)
				{
					// test the byte's highest-order bit
                    if (!oldButtons[i] && newButtons[i]) // down
					{
						if (MouseDownEvent != null)
							MouseDownEvent(this, new PolledMouseEventArgs(_state, client, screen, i));
					}
                    else if (oldButtons[i] && !newButtons[i]) // up
					{
						if (MouseUpEvent != null)
							MouseUpEvent(this, new PolledMouseEventArgs(_state, client, screen, i));
					}
				}
			}
			return true;
		}

		#endregion
	}
}
