using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using PolledDevices;

namespace PolledDevices.TestJoystick
{
	public partial class MainForm : System.Windows.Forms.Form
	{
		private PolledDevices.PolledJoystick _joystick;
		
		public MainForm()
		{
			InitializeComponent();

			// set up the joystick
			this._joystick = new PolledJoystick(this);
			_joystick.PositionThreshold = 2;
			_joystick.RotationThreshold = 2;
			_joystick.SliderThreshold = 2;

			// "register" to receive generic device events
			_joystick.DeviceAcquiredEvent += new EventHandler(OnJoystickAcquired);
			_joystick.DeviceReleasedEvent += new EventHandler(OnJoystickReleased);
			_joystick.DeviceLostEvent += new EventHandler(OnJoystickLost);

			// "register" to receive events specific to the joystick
			_joystick.JoystickPolledEvent += new PolledJoystickEventHandler(OnJoystickPolled);
			_joystick.JoystickPositionEvent += new PolledJoystickEventHandler(OnJoystickPosition);
			_joystick.JoystickRotationEvent += new PolledJoystickEventHandler(OnJoystickRotation);
			_joystick.JoystickSliderEvent += new PolledJoystickEventHandler(OnJoystickSlider);
			_joystick.JoystickPointOfViewEvent += new PolledJoystickEventHandler(OnJoystickPointOfView);
			_joystick.JoystickDownEvent += new PolledJoystickEventHandler(OnJoystickDown);
			_joystick.JoystickUpEvent += new PolledJoystickEventHandler(OnJoystickUp);
		}

        private void MainForm_Load(object sender, EventArgs e)
        {
            _joystick.Acquire();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            _joystick.Release();
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            _joystick.Release(); // ensure
        }

		private void OnJoystickAcquired(object sender, EventArgs e)
		{
			Console.WriteLine("Acquired");
		}

		private void OnJoystickReleased(object sender, EventArgs e)
		{
			Console.WriteLine("Released");
		}

		private void OnJoystickLost(object sender, EventArgs e)
		{
			Console.WriteLine("Lost");
		}

		private void OnJoystickPolled(object sender, PolledJoystickEventArgs e)
		{
			// nothing
		}

		private void OnJoystickPosition(object sender, PolledJoystickEventArgs e)
		{
			Position.Text = String.Format("({0}, {1}, {2})", e.X, e.Y, e.Z);
		}

		private void OnJoystickRotation(object sender, PolledJoystickEventArgs e)
		{
			Rotation.Text = String.Format("({0}, {1}, {2})", e.Rx, e.Ry, e.Rz);
		}

		private void OnJoystickSlider(object sender, PolledJoystickEventArgs e)
		{
			Sliders.Text = String.Format("({0}, {1})", e.Sliders.X, e.Sliders.Y);
		}

		private void OnJoystickPointOfView(object sender, PolledJoystickEventArgs e)
		{
			PointOfView.Text = String.Format("{0}", e.Pov);
		}

		private void OnJoystickDown(object sender, PolledJoystickEventArgs e)
		{
			ButtonDown.Text = String.Format("{0} at ({1}, {2}, {3})", e.Button, e.X, e.Y, e.Z);
		}

		private void OnJoystickUp(object sender, PolledJoystickEventArgs e)
		{
			ButtonUp.Text = String.Format("{0} at ({1}, {2}, {3})", e.Button, e.X, e.Y, e.Z);
		}

        
	}
}
